"""Make package extract image size column a big integer

Revision ID: b718c0f8bc53
Revises: 92a5e912f877
Create Date: 2021-12-21 11:07:37.499211+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "b718c0f8bc53"
down_revision = "92a5e912f877"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "package_extract_run", "image_size", existing_type=sa.INTEGER(), type_=sa.BigInteger(), existing_nullable=True
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "package_extract_run", "image_size", existing_type=sa.BigInteger(), type_=sa.INTEGER(), existing_nullable=True
    )
    # ### end Alembic commands ###
