"""Extend Python package index with only_packages_seen attribute

Revision ID: 65ae36e5c38a
Revises: 95c40432360f
Create Date: 2021-07-13 06:04:03.123668+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "65ae36e5c38a"
down_revision = "95c40432360f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "python_package_index", sa.Column("only_if_package_seen", sa.Boolean(), nullable=False, server_default="true")
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("python_package_index", "only_if_package_seen")
    # ### end Alembic commands ###
