"""Add platform for dependencies

Revision ID: 9542eba46906
Revises: 340d6e381d75
Create Date: 2020-06-08 07:04:08.044732+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9542eba46906"
down_revision = "340d6e381d75"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("depends_on", sa.Column("platform", sa.Text(), nullable=False, server_default="linux-x86_64"))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("depends_on", "platform")
    # ### end Alembic commands ###
