"""Add CVE timestamp table

Revision ID: 0165f03250a6
Revises: aba59a2fec95
Create Date: 2021-10-26 10:53:35.744430+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "0165f03250a6"
down_revision = "aba59a2fec95"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "cve_timestamp",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("cve_timestamp")
    # ### end Alembic commands ###
