"""Add flag for missing package version

Revision ID: ed56455efd82
Revises: 0c8eab5d4197
Create Date: 2020-02-19 19:02:38.296829+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "ed56455efd82"
down_revision = "0c8eab5d4197"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "python_package_version", sa.Column("is_missing", sa.Boolean(), nullable=False, server_default="false")
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("python_package_version", "is_missing")
    # ### end Alembic commands ###
