"""Create index depends_on extras and marker

Revision ID: abb1a0a90885
Revises: cb86ec08e5ce
Create Date: 2021-03-18 23:30:31.690394+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "abb1a0a90885"
down_revision = "cb86ec08e5ce"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "depends_on_extra_marker_evaluation_result_idx",
        "depends_on",
        ["extra", "marker_evaluation_result"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("depends_on_extra_marker_evaluation_result_idx", table_name="depends_on")
    # ### end Alembic commands ###
